<?php

namespace App\Http\Controllers;

use App\Models\Contribution;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Http\Controllers\AlertsController;
use App\Jobs\SendAlertJob;
use Barryvdh\DomPDF\Facade\Pdf;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Log;


class ContributionController extends Controller
{
    /**
     * Display a listing of contributions.
     */
    public function index(Request $request)
{
    // Fetching the page number and page size from the request
    $page = $request->input('page', 1); // Default is page 1
    $size = $request->input('size', 20); // Default page size is 20

    // Fetch contributions and join with users for member details
    $contributions = Contribution::select('contributions.*', 'users.fname', 'users.lname', 'users.image')
        ->join('users', 'contributions.memberId', '=', 'users.id')
        ->orderBy('contributions.id', 'desc') // Optional: sorting by created_at date
        ->paginate($size, ['*'], 'page', $page);

    return response()->json($contributions);
}
public function getUserContributions(Request $request, $userId)
{
    // Fetching the page number and page size from the request
    $page = $request->input('page', 1); // Default is page 1
    $size = $request->input('size', 20); // Default page size is 20

    // Fetch contributions for the specific user and join with users for member details
    $contributions = Contribution::select('contributions.*', 'users.fname', 'users.lname', 'users.image')
        ->join('users', 'contributions.memberId', '=', 'users.id')
        ->where('contributions.memberId', $userId)
        ->orderBy('contributions.id', 'desc') // Optional: sorting by created_at date
        ->paginate($size, ['*'], 'page', $page);

    return response()->json($contributions);
}


    /**
     * Store a newly created contribution in storage.
     */
    public function store(Request $request)
    {
        $validated = $request->validate([
            'memberId' => 'required|exists:users,id',  // Assuming 'users' is another table
            'amount' => 'required|numeric',
            'comment' => 'nullable|string',  // Comment can be nullable
            'mpesacode' => 'nullable|string',  // Mpesa code can be nullable
        ]);

        try {
            // If 'approved' is not provided, set it to false by default
            $contribution = Contribution::create([
                'memberId' => $validated['memberId'],
                'amount' => $validated['amount'],
                'comment' => $validated['comment'] ?? null,  // Store comment if provided
                'mpesacode' => $validated['mpesacode'] ?? null,  // Store mpesacode if provided
                'approved' => false,  // Set approved to false by default
                'approved_at' => null,
            ]);
            $admins = User::where('role', 1)->get();
            $user = User::findOrFail($contribution->memberId);
            // Prepare and dispatch an alert for each admin
            foreach ($admins as $admin) {
                $alertData = [
                    'userID' => $admin->id,
                    'title' => 'New Contribution',
                    'message' => "$user->fname has submitted contribiution of Kes $contribution->amount, please log in and approve the contribution",
                ];

                // Dispatch the SendAlertJob with a 2-second delay
                SendAlertJob::dispatch($alertData)->delay(now()->addSeconds(1));
            }
           
            return response()->json([
                'status' => 'success',
                'message' => "Contribution created successfully.\nWe will notify you when Admin approves it.",
                'contribution' => $contribution
            ], 200);
            

        } catch (\Exception $e) {
            return response()->json([
                'status' => 'error',
                'message' => 'Failed to create contribution',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    /**
     * Display the specified contribution.
     */
    public function show($id)
    {
        $contribution = Contribution::findOrFail($id);
        return response()->json($contribution);
    }

    /**
     * Update the specified contribution in storage.
     */
    public function update(Request $request,$id)
{
    $contribution = Contribution::findOrFail($id);

    // Validate the input fields, including created_at
    $validated = $request->validate([
        'amount' => 'numeric',
        'comment' => 'nullable|string',  // Comment can be nullable
        'mpesacode' => 'nullable|string',  // Mpesa code can be nullable
        'created_at' => 'nullable|date',  // Add validation for created_at
    ]);

    try {
        // Update the contribution, including created_at if provided
        $contribution->update([
            'amount' => $validated['amount'] ?? $contribution->amount,
            'comment' => $validated['comment'] ?? $contribution->comment,  // Update comment if provided
            'mpesacode' => $validated['mpesacode'] ?? $contribution->mpesacode,  // Update mpesacode if provided
            'created_at' => $validated['created_at'] ?? $contribution->created_at, 
           'updated_at'=>now(),// Update created_at if provided
        ]);
        $user = User::findOrFail($contribution->memberId);
        $alertData = [
            'userID' => $user->id,
            'title' => 'Contribution Updates',
            'message' => "Contribution details have been updated",
        ];

        // Dispatch the SendAlertJob with a 2-second delay
        SendAlertJob::dispatch($alertData)->delay(now()->addSeconds(1));

        return response()->json([
            'status' => 'success',
            'message' => "Contribution updated successfully.",
            'contribution' => $contribution
        ], 200);

    } catch (\Exception $e) {
        return response()->json([
            'status' => 'error',
            'message' => 'Failed to update contribution',
            'error' => $e->getMessage(),
        ], 500);
    }
}


    /**
     * Remove the specified contribution from storage.
     */
    public function destroy($id)
    {
        $contribution = Contribution::findOrFail($id);
        $contribution->delete();

        return response()->json([
            'status' => 'success',
            'message' => 'Contribution deleted successfully',
        ], 200);
    }
//   public function updateApprovalStatus(Request $request, $id)
// {
//     // Validate the incoming request to ensure 'status' is provided and is either 0 or 1
//     $request->validate([
//         'status' => 'required|in:0,1',
//     ]);

//     try {
//         // Find the contribution by ID
//         $contribution = Contribution::findOrFail($id);
        
//         // Update the approval status
//         $contribution->approved = $request->status;
//         $contribution->approved_at = now();
        
//         $contribution->save();

//         // Retrieve the user associated with the contribution
        

//         // Determine the message based on the status
//         $message = $request->status == 1 
//             ? 'Contribution approved successfully.'
//             : 'Contribution set to Pending.';

//         // Prepare alert data
//         $user = User::findOrFail($contribution->memberId);
//         $alertData = [
//             'userID' => $user->id,
//             'title' => 'Contribution Status',
//             'message' => $message,
//         ];

//         // Dispatch the SendAlertJob with a 2-second delay
//         SendAlertJob::dispatch($alertData)->delay(now()->addSeconds(1));

//         // Return a success response with an accurate message
//         return response()->json([
//             'status' => 'success',
//             'message' => $message,
//             'contribution' => $contribution
//         ], 200);

//     } catch (\Exception $e) {
//         // Handle error and return a failure response
//         return response()->json([
//             'status' => 'error',
//             'message' => 'Failed to update contribution approval status.',
//             'error' => $e->getMessage(),
//         ], 500);
//     }
// }

public function updateApprovalStatus(Request $request, $id)
{
    // Validate the incoming request to ensure 'status' is provided and is either 0 or 1
    $request->validate([
        'status' => 'required|in:0,1',
    ]);

    try {
        // Find the contribution by ID
        $contribution = Contribution::findOrFail($id);
        
        // Update the approval status
        $contribution->approved = $request->status;
        $contribution->approved_at = now();
        $contribution->save();

        // Retrieve the user associated with the contribution (this is the contributor)
        $user = User::findOrFail($contribution->memberId);

        // Determine the message based on the status
        $message = $request->status == 1 
            ? 'Contribution approved successfully. A receipt has been emailed to the client.'
            : 'Contribution set to Pending.';

        // Prepare alert data for user notification
        $alertData = new Request([
            'userID' => $user->id,
            'title' => 'Contribution Status',
            'message' => $message,
        ]);
        
         

        // Dispatch the SendAlertJob with a 2-second delay
        //SendAlertJob::dispatch($alertData)->delay(now()->addSeconds(1));
         (new AlertsController())->sendAlert($alertData);

        // Generate PDF for share purchase receipt if the contribution is approved
        if ($request->status == 1) {
            // Load the PDF view and pass the required data (contribution details and user details)
            $pdf = Pdf::loadView('emails.contribution_receipt', ['contribution' => $contribution, 'user' => $user]);

            $pdfOutput = $pdf->output(); // Get the PDF output as a string

            // Send email with attached PDF (in memory) to the contributor
            try {
                Mail::send('emails.contribution_receipt', ['user' => $user,'contribution' => $contribution], function ($message) use ($user, $pdfOutput, $contribution) {
                    $message->to($user->email) // Contributor's email
                        ->subject('Contribution Receipt for Purchase of Shares')
                        ->attachData($pdfOutput, 'contribution_receipt_' . $contribution->id . '.pdf', [
                            'mime' => 'application/pdf',
                        ]); // Attach PDF from memory
                });
                Log::info('Contribution receipt email with PDF sent successfully to ' . $user->email);
            } catch (\Exception $e) {
                // Log the error for debugging
                Log::error('Failed to send contribution receipt email with PDF: ' . $e->getMessage());
            }
        }

        // Return a success response with an accurate message
        return response()->json([
            'status' => 'success',
            'message' => $message,
            'contribution' => $contribution,
        ], 200);

    } catch (\Exception $e) {
        // Handle error and return a failure response
        return response()->json([
            'status' => 'error',
            'message' => 'Failed to update contribution approval status.',
            'error' => $e->getMessage(),
        ], 200);
    }
}



}
