<?php

namespace App\Http\Controllers;
use App\Models\Loan;
use Illuminate\Http\Request;
use App\Models\User;
use Carbon\Carbon;
use Illuminate\Support\Facades\Auth;
use App\Models\Contribution;
use App\Http\Controllers\AlertsController;
use App\Models\Guarantor;
use Illuminate\Support\Facades\DB;  // For database transactions
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Mail;
use Barryvdh\DomPDF\Facade\Pdf;


class LoanController extends Controller
{
    /**
     * Display a listing of the loans.
     */
    public function index()
    {
        $loans = Loan::all();
        return response()->json($loans);  // Return all loans as JSON
    }

    /**
     * Store a newly created loan in storage.
     */
public function store(Request $request)
{
    // Validate incoming request
    $validated = $request->validate([
        'type' => 'required|string',
        'amount' => 'required|numeric',
        'memberId' => 'required|exists:users,id',  // Ensure memberId exists in users table
        'duration' => 'required|numeric|min:1|max:12',  // Validate loan duration
        'comments' => 'nullable|string',  // Comments can be optional
        'totalRepayment' => 'required|numeric',  // Total repayment must be provided
        //'guarantors' => 'nullable|array',  // Guarantors can be optional
        //'guarantors.*.id' => 'required_with:guarantors|numeric',  // Validate guarantor name if provided
       // 'guarantors.*.amount' => 'required_with:guarantors|numeric',  // Validate guarantor amount if provided
    ]);

    // Fetch user details based on memberId
    $user = User::findOrFail($validated['memberId']);
    
    // Generate Loan ID
    $firstLetterFname = strtoupper(substr($user->fname, 0, 1));
    $firstLetterLname = strtoupper(substr($user->lname, 0, 1));
    $loanCount = Loan::count();
    $month = date('m');
    $year = date('y');
    $loanId = $firstLetterFname . $firstLetterLname . '-' .$year.$month.($loanCount + 1);

    // Begin database transaction
    DB::beginTransaction();

    try {
        // Create new loan
        $loan = Loan::create([
            'loanId' => $loanId,
            'type' => $validated['type'],
            'amount' => $validated['amount'],
            'memberId' => $validated['memberId'],
            'duration' => $validated['duration'],
            'comments' => $validated['comments'],
            'totalRepayment' => $validated['totalRepayment'],
        ]);
           
        // Handle guarantors if provided
        if (isset($request->guarantors)) {
    foreach ($request->guarantors as $guarantor) {
        // Create the guarantor record
        $guarantorRec = Guarantor::create([
            'loan_id' => $loan->loanId,  // Reference the created loan
            'guarantor_id' => $guarantor['guarantor_id'],  // Guarantor's id
            'amount' => $guarantor['amount'],  // Amount the guarantor is responsible for
        ]);

        // Retrieve the guarantor's user information
        $guar = User::where('id', $guarantor['guarantor_id'])->first();

        // Ensure $guar is found before proceeding
        if ($guar) {
            // Prepare the message for the guarantor
            $guarantorMessage = new Request([
                'userID' =>  $guar->id,
                'title' => 'Guarantee Request',
                'message' => "Hi {$guar->fname},\n{$user->fname} {$user->lname} has requested you to guarantee the #{$loan->loanId} for the amount of KES {$loan->amount} with KES {$guarantor['amount']}. Please log in to review the request.",
            ]);

            // Send the notification to the guarantor
            (new AlertsController())->sendAlert($guarantorMessage);
        }
    }
}


        // Commit the transaction
        DB::commit();

        // Return success response
        return response()->json([
            'status' => 'success',
          'message' => "Loan ID: " . $loan->loanId . "\nLoan created successfully. We will notify you once the Guarantors accept your request. You will get the money once the Admin approves your loan.",


            'loan' => $loan,
        ], 200);

    } catch (\Exception $e) {
        // Rollback the transaction if there is an error
        DB::rollBack();

        // Return error response
        return response()->json([
            'status' => 'error',
            'message' => 'Failed to create loan. Please try again.',
            'error' => $e->getMessage(),
        ], 500);
    }
}




    /**
     * Display the specified loan.
     */
    public function show($id)
    {
        // Find the loan by id or fail
        $loan = Loan::findOrFail($id);

        return response()->json($loan);
    }

    /**
     * Update the specified loan in storage.
     */
    public function update(Request $request, $id)
    {
        // Find the loan by id
        $loan = Loan::findOrFail($id);

        // Validate incoming request
        $validated = $request->validate([
            'loanId' => 'sometimes|required|unique:loans,loanId,' . $loan->id,
            'type' => 'sometimes|required|string',
            'amount' => 'sometimes|required|numeric',
            'memberId' => 'sometimes|required|exists:members,id',
        ]);

        // Update the loan with the validated data
        $loan->update($validated);

        return response()->json([
            'status' => 'success',
            'message' => 'Loan updated successfully',
            'loan' => $loan
        ], 200);
    }

    /**
     * Remove the specified loan from storage.
     */
    public function destroy($id)
    {
        // Find the loan by id
        $loan = Loan::findOrFail($id);

        // Delete the loan
        $loan->delete();

        return response()->json([
            'status' => 'success',
            'message' => 'Loan deleted successfully'
        ], 200);
    }



    public function getApplicantInfo($id)
    {
        // Fetch user details from the users table
        $user = User::find($id);

        if (!$user) {
            return response()->json([
                'status' => 'error',
                'message' => 'User not found'
            ], 404);
        }

        // Calculate the total contributions from the contributions table
        $totalContributions = Contribution::where('memberId', $id) ->where('approved', true)->sum('amount');

        // Return user details along with total contributions
        return response()->json([
            'status' => 'success',
            'user' => [
                'id' => $user->id,
                'fname' => $user->fname,
                'lname' => $user->lname,
                'email' => $user->email,
                'phone' => $user->phone,
                'IdNumber' => $user->IdNumber,
                'shares' => $totalContributions, // Total of contributions as shares
            ]
        ]);
    }
    // Method to fetch pending loan requests by userID
 





public function getPendingLoans($userID)
{
    try {
        // Fetch the user (loan applicant) details
        $user = User::find($userID);

        if (!$user) {
            return response()->json([
                'message' => 'User not found'
            ], 404);
        }

        // Fetch loans with their guarantor details and the new admin approval fields
        $pendingLoans = DB::table('loans')
            ->leftJoin('loan_guarantors', 'loans.loanId', '=', 'loan_guarantors.loan_id')
            ->where('loans.memberId', $userID)
            ->where('loans.admin1_approved', false)
            ->where('loans.admin2_approved', false)
            ->select(
                'loans.id as loan_id',
                'loans.loanId',
                'loans.amount',
                'loans.type',
                'loans.totalRepayment',
                'loans.duration',
                'loans.created_at',
                'loans.updated_at',
                'loans.memberId',
                'loans.admin1_approved',
                'loans.admin2_approved',
                'loans.admin1_approved_at',
                'loans.admin2_approved_at',
                'loans.admin1_approved_by',
                'loans.admin2_approved_by',
                'loan_guarantors.guarantor_id',
                'loan_guarantors.accepted',
                'loan_guarantors.accepted_at'
            )
            ->get();

        // Group the loans and guarantor details
        $groupedLoans = [];
        foreach ($pendingLoans as $loan) {
            if (!isset($groupedLoans[$loan->loan_id])) {
                // Initialize the loan with the new admin approval fields and add user (applicant) name
                $groupedLoans[$loan->loan_id] = [
                    'loan_id' => $loan->loan_id,
                    'loanId' => $loan->loanId,
                    'amount' => $loan->amount,
                    'type' => $loan->type,
                    'totalRepayment' => $loan->totalRepayment,
                    'duration' => $loan->duration,
                    'created_at' => $loan->created_at,
                    'updated_at' => $loan->updated_at,
                    'memberId' => $loan->memberId,
                    'admin1_approved' => $loan->admin1_approved,
                    'admin2_approved' => $loan->admin2_approved,
                    'admin1_approved_at' => $loan->admin1_approved_at,
                    'admin2_approved_at' => $loan->admin2_approved_at,
                    'admin1_approved_by' => $loan->admin1_approved_by,
                    'admin2_approved_by' => $loan->admin2_approved_by,
                    'user_name' => $user->fname . ' ' . $user->lname,  // Include applicant's full name
                    'guarantors' => []
                ];
            }

            // Add guarantor details if they exist
            if ($loan->guarantor_id) {
                $guarantor = User::find($loan->guarantor_id);
                if ($guarantor) {
                    $groupedLoans[$loan->loan_id]['guarantors'][] = [
                        'guarantor_id' => $loan->guarantor_id,
                        'guarantor_name' => $guarantor->fname . ' ' . $guarantor->lname,  // Join first and last name
                        'accepted' => $loan->accepted,
                        'accepted_at' => $loan->accepted_at,
                    ];
                }
            }
        }

        // Convert associative array to indexed array for proper JSON response
        $groupedLoans = array_values($groupedLoans);

        return response()->json($groupedLoans, 200);
    } catch (\Exception $e) {
        return response()->json([
            'message' => 'Failed to fetch pending loans',
            'error' => $e->getMessage()
        ], 500);
    }
}



// public function getUserLoans(Request $request, $userID)
// {
//     try {
//         // Get pagination parameters with default values
//         $perPage = $request->input('per_page', 20); // Number of items per page
//         $page = $request->input('page', 1); // Current page

//         // Retrieve the user associated with the userID
//         $user = User::find($userID);

//         if (!$user) {
//             return response()->json([
//                 'message' => 'User not found'
//             ], 404);
//         }

//         // Fetch loans with their guarantor details
//         $loansQuery = DB::table('loans')
//             ->leftJoin('loan_guarantors', 'loans.loanId', '=', 'loan_guarantors.loan_id')
//             ->where('loans.memberId', $userID)
//             ->where('loans.approved', true)
//             ->select(
//                 'loans.id as loan_id',
//                 'loans.loanId',
//                 'loans.amount',
//                 'loans.type',
//                 'loans.totalRepayment',
//                 'loans.duration',
//                 'loans.created_at',
//                 'loans.updated_at',
//                 'loans.memberId',
//                 'loans.approved',
//                 'loans.approved_at',
//                 'loan_guarantors.guarantor_id',
//                 'loan_guarantors.accepted',
//                 'loan_guarantors.accepted_at'
//             );

//         // Apply pagination
//         $loansPaginated = $loansQuery->paginate($perPage, ['*'], 'page', $page);

//         // Group the loans and guarantor details
//         $groupedLoans = [];
//         foreach ($loansPaginated->items() as $loan) {
//             if (!isset($groupedLoans[$loan->loan_id])) {
//                 // Initialize the loan with empty guarantors array and include the user's name
//                 $groupedLoans[$loan->loan_id] = [
//                     'loan_id' => $loan->loan_id,
//                     'loanId' => $loan->loanId,
//                     'amount' => $loan->amount,
//                     'type' => $loan->type,
//                     'totalRepayment' => $loan->totalRepayment,
//                     'duration' => $loan->duration,
//                     'created_at' => $loan->created_at,
//                     'updated_at' => $loan->updated_at,
//                     'memberId' => $loan->memberId,
//                     'approved' => $loan->approved,
//                     'approved_at' => $loan->approved_at,
//                     'user_name' => $user->fname . ' ' . $user->lname,  // Include the user's full name in the loan details
//                     'guarantors' => []
//                 ];
//             }

//             // Add guarantor details if they exist
//             if ($loan->guarantor_id) {
//                 $guarantor = User::find($loan->guarantor_id);
//                 if ($guarantor) {
//                     $groupedLoans[$loan->loan_id]['guarantors'][] = [
//                         'guarantor_id' => $loan->guarantor_id,
//                         'guarantor_name' => $guarantor->fname . ' ' . $guarantor->lname, // Correctly joining fname and lname
//                         'accepted' => $loan->accepted,
//                         'accepted_at' => $loan->accepted_at,
//                     ];
//                 }
//             }
//         }

//         // Convert associative array to indexed array for proper JSON response
//         $groupedLoans = array_values($groupedLoans);

//         // Return paginated response with grouped loans
//         $response = [
//             'current_page' => $loansPaginated->currentPage(),
//             'data' => $groupedLoans,
//             'first_page_url' => $loansPaginated->url(1),
//             'from' => $loansPaginated->firstItem(),
//             'last_page' => $loansPaginated->lastPage(),
//             'last_page_url' => $loansPaginated->url($loansPaginated->lastPage()),
//             'next_page_url' => $loansPaginated->nextPageUrl(),
//             'path' => $loansPaginated->path(),
//             'per_page' => $loansPaginated->perPage(),
//             'prev_page_url' => $loansPaginated->previousPageUrl(),
//             'to' => $loansPaginated->lastItem(),
//             'total' => $loansPaginated->total(),
//         ];

//         return response()->json($response, 200);

//     } catch (\Exception $e) {
//         return response()->json([
//             'message' => 'Failed to fetch loans',
//             'error' => $e->getMessage()
//         ], 500);
//     }
// }

public function getUserLoans(Request $request, $userID)
{
    try {
        // Get pagination parameters with default values
        $perPage = $request->input('per_page', 20); // Number of items per page
        $page = $request->input('page', 1); // Current page

        // Retrieve the user associated with the userID
        $user = User::find($userID);

        if (!$user) {
            return response()->json([
                'message' => 'User not found'
            ], 404);
        }

        // Fetch loans with their guarantor details and calculate loanbalance
        $loansQuery = DB::table('loans')
            ->leftJoin('loan_guarantors', 'loans.loanId', '=', 'loan_guarantors.loan_id')
            ->where('loans.memberId', $userID)
            ->where('loans.admin1_approved', true)
            ->where('loans.admin2_approved', true)
            ->select(
                'loans.id as loan_id',
                'loans.loanId',
                'loans.amount',
                'loans.type',
                'loans.totalRepayment',
                'loans.duration',
                'loans.created_at',
                'loans.updated_at',
                'loans.memberId',
                'loans.admin1_approved',
                'loans.admin2_approved',
                'loans.admin1_approved_at',
                'loans.admin2_approved_at',
                'loans.admin1_approved_by',
                'loans.admin2_approved_by',
                'loan_guarantors.guarantor_id',
                'loan_guarantors.accepted',
                'loan_guarantors.accepted_at',
                // Calculate loanbalance: totalRepayment - sum of approved loan payments
                DB::raw('(loans.totalRepayment - IFNULL(
                    (SELECT SUM(amount) FROM loanpayments WHERE loanpayments.loanId = loans.loanId AND loanpayments.approved = true), 
                    0
                )) AS loanbalance')
            );

        // Apply pagination
        $loansPaginated = $loansQuery->paginate($perPage, ['*'], 'page', $page);

        // Group the loans and guarantor details
        $groupedLoans = [];
        foreach ($loansPaginated->items() as $loan) {
            if (!isset($groupedLoans[$loan->loan_id])) {
                // Initialize the loan with empty guarantors array and include the user's name
                $groupedLoans[$loan->loan_id] = [
                    'loan_id' => $loan->loan_id,
                    'loanId' => $loan->loanId,
                    'amount' => $loan->amount,
                    'type' => $loan->type,
                    'totalRepayment' => $loan->totalRepayment,
                    'duration' => $loan->duration,
                    'created_at' => $loan->created_at,
                    'updated_at' => $loan->updated_at,
                    'memberId' => $loan->memberId,
                    'admin1_approved' => $loan->admin1_approved,
                    'admin2_approved' => $loan->admin2_approved,
                    'admin1_approved_at' => $loan->admin1_approved_at,
                    'admin2_approved_at' => $loan->admin2_approved_at,
                    'admin1_approved_by' => $loan->admin1_approved_by,
                    'admin2_approved_by' => $loan->admin2_approved_by,
                    'loanbalance' => $loan->loanbalance, // Include loanbalance
                    'user_name' => $user->fname . ' ' . $user->lname,  // Include the user's full name in the loan details
                    'guarantors' => []
                ];
            }

            // Add guarantor details if they exist
            if ($loan->guarantor_id) {
                $guarantor = User::find($loan->guarantor_id);
                if ($guarantor) {
                    $groupedLoans[$loan->loan_id]['guarantors'][] = [
                        'guarantor_id' => $loan->guarantor_id,
                        'guarantor_name' => $guarantor->fname . ' ' . $guarantor->lname, // Correctly joining fname and lname
                        'accepted' => $loan->accepted,
                        'accepted_at' => $loan->accepted_at,
                    ];
                }
            }
        }

        // Convert associative array to indexed array for proper JSON response
        $groupedLoans = array_values($groupedLoans);

        // Return paginated response with grouped loans
        $response = [
            'current_page' => $loansPaginated->currentPage(),
            'data' => $groupedLoans,
            'first_page_url' => $loansPaginated->url(1),
            'from' => $loansPaginated->firstItem(),
            'last_page' => $loansPaginated->lastPage(),
            'last_page_url' => $loansPaginated->url($loansPaginated->lastPage()),
            'next_page_url' => $loansPaginated->nextPageUrl(),
            'path' => $loansPaginated->path(),
            'per_page' => $loansPaginated->perPage(),
            'prev_page_url' => $loansPaginated->previousPageUrl(),
            'to' => $loansPaginated->lastItem(),
            'total' => $loansPaginated->total(),
        ];

        return response()->json($response, 200);

    } catch (\Exception $e) {
        return response()->json([
            'message' => 'Failed to fetch loans',
            'error' => $e->getMessage()
        ], 500);
    }
}


public function getAllLoans(Request $request)
{
    try {
        // Get pagination parameters with default values
        $perPage = $request->input('per_page', 20); // Number of items per page
        $page = $request->input('page', 1); // Current page

        // Fetch loans with their guarantor details
        $loansQuery = DB::table('loans')
            ->leftJoin('loan_guarantors', 'loans.loanId', '=', 'loan_guarantors.loan_id')
            ->leftJoin('users', 'loans.memberId', '=', 'users.id') // Join with users table to get the user (loan applicant)
            ->select(
                'loans.id as loan_id',
                'loans.loanId',
                'loans.amount',
                'loans.type',
                'loans.totalRepayment',
                'loans.duration',
                'loans.created_at',
                'loans.updated_at',
                'loans.memberId',
                'loans.admin1_approved',
                'loans.admin2_approved',
                'loans.admin1_approved_at',
                'loans.admin2_approved_at',
                'loans.admin1_approved_by',
                'loans.admin2_approved_by',
                'loan_guarantors.guarantor_id',
                'loan_guarantors.accepted',
                'loan_guarantors.accepted_at',
                'loan_guarantors.amount as guarantor_amount',
                'users.fname as user_fname', // Get user's first name
                'users.lname as user_lname'  // Get user's last name
            );

        // Apply pagination
        $loansPaginated = $loansQuery->paginate($perPage, ['*'], 'page', $page);

        // Group the loans and guarantor details
        $groupedLoans = [];
        foreach ($loansPaginated->items() as $loan) {
            if (!isset($groupedLoans[$loan->loan_id])) {
                // Initialize the loan with empty guarantors array
                $groupedLoans[$loan->loan_id] = [
                    'loan_id' => $loan->loan_id,
                    'loanId' => $loan->loanId,
                    'amount' => $loan->amount,
                    'type' => $loan->type,
                    'totalRepayment' => $loan->totalRepayment,
                    'duration' => $loan->duration,
                    'created_at' => $loan->created_at,
                    'updated_at' => $loan->updated_at,
                    'memberId' => $loan->memberId,
                    'admin1_approved' => $loan->admin1_approved,
                    'admin2_approved' => $loan->admin2_approved,
                    'admin1_approved_at' => $loan->admin1_approved_at,
                    'admin2_approved_at' => $loan->admin2_approved_at,
                    'admin1_approved_by' => $loan->admin1_approved_by,
                    'admin2_approved_by' => $loan->admin2_approved_by,
                    'user_name' => $loan->user_fname . ' ' . $loan->user_lname, // Combine user's first and last name
                    'guarantors' => []
                ];
            }

            // Add guarantor details if they exist
            if ($loan->guarantor_id) {
                $guarantor = User::find($loan->guarantor_id);
                if ($guarantor) {
                    $groupedLoans[$loan->loan_id]['guarantors'][] = [
                        'guarantor_id' => $loan->guarantor_id,
                        'guarantor_name' => $guarantor->fname . ' ' . $guarantor->lname, 
                        'guarantor_amount' => $loan->guarantor_amount,
                        'accepted' => $loan->accepted,
                        'accepted_at' => $loan->accepted_at,
                    ];
                }
            }
        }

        // Convert associative array to indexed array for proper JSON response
        $groupedLoans = array_values($groupedLoans);

        // Return paginated response with grouped loans
        $response = [
            'current_page' => $loansPaginated->currentPage(),
            'data' => $groupedLoans,
            'first_page_url' => $loansPaginated->url(1),
            'from' => $loansPaginated->firstItem(),
            'last_page' => $loansPaginated->lastPage(),
            'last_page_url' => $loansPaginated->url($loansPaginated->lastPage()),
            'next_page_url' => $loansPaginated->nextPageUrl(),
            'path' => $loansPaginated->path(),
            'per_page' => $loansPaginated->perPage(),
            'prev_page_url' => $loansPaginated->previousPageUrl(),
            'to' => $loansPaginated->lastItem(),
            'total' => $loansPaginated->total(),
        ];

        return response()->json($response, 200);

    } catch (\Exception $e) {
        return response()->json([
            'message' => 'Failed to fetch loans',
            'error' => $e->getMessage()
        ], 500);
    }
}


    // public function approveLoan($loanId)
    // {
    //     try {
    //         // Find the loan by loanId from the 'loans' table
    //         $loan = Loan::where('loanId', $loanId)->first();

    //         if (!$loan) {
    //             return response()->json([
    //                 'status' => 'error',
    //                 'message' => 'Loan not found.',
    //             ], 404);
    //         }

    //         // Fetch the user (member) associated with the loan
    //         $user = User::find($loan->memberId); // Assuming memberId is the user ID

    //         if (!$user) {
    //             return response()->json([
    //                 'status' => 'error',
    //                 'message' => 'User not found for this loan.',
    //             ], 404);
    //         }

    //         // Check if all guarantors have approved
    //         $allGuarantorsApproved = collect($loan->guarantors)->every(function ($guarantor) {
    //             return $guarantor['accepted'] == true;
    //         });

    //         if (!$allGuarantorsApproved) {
    //             return response()->json([
    //                 'status' => 'error',
    //                 'message' => 'Not all guarantors have approved.',
    //             ], 400);
    //         }

    //         // Approve the loan
    //         $loan->approved = true;
    //         $loan->approved_at = now();
    //         $loan->save();
            
    //           $alertData = new Request([
    //             'userID' =>  $user->id,
    //             'title' => 'Loan Approval',
    //             'message' => "Your loan #$loan->loanId has been approved by the admin",
    //         ]);
    //         (new AlertsController())->sendAlert($alertData);

    //         return response()->json([
    //             'status' => 'success',
    //             'message' => 'Loan approved successfully.',
    //             'loan' => $loan,
    //             'user' => $user, // Returning user data in the response
    //         ], 200);
    //     } catch (\Exception $e) {
    //         return response()->json([
    //             'status' => 'error',
    //             'message' => 'Failed to approve the loan. Please try again.',
    //         ], 500);
    //     }
    // }
  

public function approveLoan($loanId, $adminId)
{
    try {
        // Find the loan by loanId from the 'loans' table
        $loan = Loan::where('loanId', $loanId)->first();

        if (!$loan) {
            return response()->json([
                'status' => 'error',
                'message' => 'Loan not found.',
            ], 200);
        }

        // Fetch the user (member) associated with the loan
        $user = User::find($loan->memberId);

        if (!$user) {
            return response()->json([
                'status' => 'error',
                'message' => 'User not found for this loan.',
            ], 200);
        }

                // Initialize message
        $message = "";

        // Check which admin is approving the loan based on adminId
        if (!$loan->admin1_approved && $loan->admin1_approved_by == null) {
            // Approve as admin1 and store adminId
            $loan->admin1_approved = true;
            $loan->admin1_approved_at = now();
            $loan->admin1_approved_by = $adminId; // Store the adminId for admin1
            $message = "Your loan #$loan->loanId has been approved by the first admin.";
        } elseif (!$loan->admin2_approved && $loan->admin2_approved_by == null) {
            // Approve as admin2 and store adminId
            $loan->admin2_approved = true;
            $loan->admin2_approved_at = now();
            $loan->admin2_approved_by = $adminId; // Store the adminId for admin2
            $message = "Your loan #$loan->loanId has been approved by the second admin.";
        } else {
            return response()->json([
                'status' => 'error',
                'message' => 'Loan is already approved by the specified admin or both admins have approved.',
            ], 200);
        }

        // Check if both admins have approved, then mark the loan as fully approved
        if ($loan->admin1_approved && $loan->admin2_approved) {
           
            $message = "Your loan #$loan->loanId has been fully approved by both admins.";

            // Only send the email when both admins have approved
            // Generate PDF in memory for email attachment
            $pdf = Pdf::loadView('emails.loan_approval', ['loan' => $loan, 'user' => $user]);
            $pdfOutput = $pdf->output(); // Get the PDF output as a string

            // Send email with attached PDF
            try {
                Mail::send('emails.loan_approval', ['loan' => $loan, 'user' => $user], function ($message) use ($user, $loan, $pdfOutput) {
                    $message->to($user->email)
                        ->subject('Loan Approved: ' . $loan->loanId)
                        ->attachData($pdfOutput, 'loan_approval_' . $loan->loanId . '.pdf', [
                            'mime' => 'application/pdf',
                        ]); // Attach PDF from memory
                });

                Log::info('Loan approval email with PDF sent successfully to ' . $user->email);
            } catch (\Exception $e) {
                // Log the error for debugging
                Log::error('Failed to send loan approval email with PDF: ' . $e->getMessage());

                return response()->json([
                    'status' => 'error',
                    'message' => 'Failed to send loan approval email with PDF. ' . $e->getMessage(),
                    'error' => $e->getMessage(),
                ], 200);
            }
        }

        // Save the loan approval status
        $loan->save();

        // Send alert to the user with the appropriate message
        $alertData = new Request([
            'userID' =>  $user->id,
            'title' => 'Loan Approval',
            'message' => $message,
        ]);
        (new AlertsController())->sendAlert($alertData);

        return response()->json([
            'status' => 'success',
            'message' => 'Loan approved successfully.',
            'loan' => $loan,
            'user' => $user,
        ], 200);

    } catch (\Exception $e) {
        Log::error('Failed to approve loan: ' . $e->getMessage());

        return response()->json([
            'status' => 'error',
            'message' => 'Failed to approve the loan. Please try again.',
            'error' => $e->getMessage(),
        ], 500);
    }
}



    // guarantoship


    /**
     * Fetches loans the authenticated user has guaranteed and loans they are guaranteed for.
     *
     * @return \Illuminate\Http\JsonResponse
     */
// public function getLoanDashboardData($userId)
// {
//     // Ensure the user is authenticated
//     if (!$userId) {
//         return response()->json(['error' => 'Unauthorized'], 401);
//     }

//     // Fetch pagination parameters from the request, with defaults
//     $page = request()->get('page', 1);
//     $limit = request()->get('limit', 10);  // Default limit is 10 items per page
//     $offset = ($page - 1) * $limit;

//     /**
//      * 1. Loans I Have Guaranteed
//      *    - Fetch loans where the user is a guarantor.
//      *    - Join with loans and users to get borrower details and loan details.
//      *    - Also join guarantors to fetch guarantor details for each loan.
//      */
//     $loansGuaranteedQuery = DB::table('loan_guarantors')
//         ->where('loan_guarantors.guarantor_id', $userId)
//         ->join('loans', 'loan_guarantors.loan_id', '=', 'loans.loanId')
//         ->join('users', 'loans.memberId', '=', 'users.id')  // Join users for borrower details
//         ->leftJoin('loan_guarantors as other_guarantors', 'loans.loanId', '=', 'other_guarantors.loan_id')
//         ->leftJoin('users as guarantor_users', 'other_guarantors.guarantor_id', '=', 'guarantor_users.id') // Join users for guarantor details
//         ->select(
//             'loans.id as loan_id',
//             'loans.loanId as loan_id_display',
//             'loans.type',
//             'loans.amount',
//             'loans.totalRepayment',
//             'loans.duration',
//             'loans.memberId',
//             'users.fname as member_first_name',
//             'users.lname as member_last_name',
//             'users.image as member_image', // Borrower name and image
//             'loans.approved',
//             'loans.approved_at',
//             'loans.created_at',
//             'loans.updated_at',
//             'loan_guarantors.id as guarantor_entry_id',
//             'loan_guarantors.amount as guaranteed_amount',
//             'loan_guarantors.accepted',
//             'loan_guarantors.accepted_at',
//             'loan_guarantors.created_at as guarantor_created_at',
//             'loan_guarantors.updated_at as guarantor_updated_at',
//             'other_guarantors.guarantor_id as other_guarantor_id',
//             'guarantor_users.fname as guarantor_first_name',
//             'guarantor_users.lname as guarantor_last_name',
//             'guarantor_users.image as guarantor_image',  // Other guarantors' name and image
//             'other_guarantors.amount as guarantor_guaranteed_amount',
//             'other_guarantors.accepted as guarantor_accepted',
//             'other_guarantors.accepted_at as guarantor_accepted_at',
//             'other_guarantors.created_at as guarantor_created_at',
//             'other_guarantors.updated_at as guarantor_updated_at'
//         )
//         ->orderBy('loans.created_at', 'desc');

//     // Get the total count for pagination
//     $totalLoansGuaranteed = $loansGuaranteedQuery->count();

//     // Apply pagination
//     $loansGuaranteed = $loansGuaranteedQuery
//         ->offset($offset)
//         ->limit($limit)
//         ->get();

//     /**
//      * 2. Group the loans by loan ID to aggregate guarantors
//      */
//     $loansGuaranteedGrouped = [];

//     foreach ($loansGuaranteed as $loan) {
//         $loanId = $loan->loan_id_display;

//         if (!isset($loansGuaranteedGrouped[$loanId])) {
//             $loansGuaranteedGrouped[$loanId] = [
//                 'id' => $loan->loan_id,
//                 'loan_id' => $loanId,
//                 'type' => $loan->type,
//                 'amount' => $loan->amount,
//                 'totalRepayment' => $loan->totalRepayment,
//                 'duration' => $loan->duration,
//                 'memberId' => $loan->memberId,
//                 'member_name' => "{$loan->member_first_name} {$loan->member_last_name}",
//                 'member_image' => $loan->member_image,  // Borrower's profile image
//                 'approved' => $loan->approved,
//                 'approved_at' => $loan->approved_at,
//                 'created_at' => $loan->created_at,
//                 'updated_at' => $loan->updated_at,
//                 'guarantors' => [],
//             ];
//         }

//         // Add each guarantor for this loan
//         if ($loan->other_guarantor_id && $loan->other_guarantor_id != $userId) {
//             $loansGuaranteedGrouped[$loanId]['guarantors'][] = [
//                 'guarantor_id' => $loan->other_guarantor_id,
//                 'guarantor_name' => "{$loan->guarantor_first_name} {$loan->guarantor_last_name}",
//                 'guarantor_image' => $loan->guarantor_image,
//                 'guaranteed_amount' => $loan->guarantor_guaranteed_amount,
//                 'accepted' => $loan->guarantor_accepted,
//                 'accepted_at' => $loan->guarantor_accepted_at,
//                 'created_at' => $loan->guarantor_created_at,
//                 'updated_at' => $loan->guarantor_updated_at,
//             ];
//         }
//     }

//     // Convert the grouped loans into a list
//     $loansGuaranteedList = array_values($loansGuaranteedGrouped);

//     /**
//      * 3. Loans I Am Guaranteed
//      *    - Fetch loans where the user is the borrower (memberId).
//      *    - Join with loan_guarantors and users to get guarantor details.
//      */
//     $loansBeingGuaranteedQuery = DB::table('loans')
//         ->where('loans.memberId', $userId)
//         ->join('loan_guarantors', 'loans.loanId', '=', 'loan_guarantors.loan_id')
//         ->join('users', 'loans.memberId', '=', 'users.id')  // Join users for member details
//         ->select(
//             'loans.id as loan_id',
//             'loans.loanId as loan_id_display',
//             'loans.type',
//             'loans.amount',
//             'loans.totalRepayment',
//             'loans.duration',
//             'loans.memberId',
//             'users.fname as member_first_name',
//             'users.lname as member_last_name',
//             'users.image as member_image',  // Member name and image
//             'loans.approved',
//             'loans.approved_at',
//             'loans.created_at',
//             'loans.updated_at',
//             'loan_guarantors.guarantor_id',
//             'loan_guarantors.amount as guaranteed_amount',
//             'loan_guarantors.accepted',
//             'loan_guarantors.accepted_at',
//             'loan_guarantors.created_at as guarantor_created_at',
//             'loan_guarantors.updated_at as guarantor_updated_at'
//         );

//     // Get the total count for pagination
//     $totalLoansBeingGuaranteed = $loansBeingGuaranteedQuery->count();

//     // Apply pagination
//     $loansBeingGuaranteed = $loansBeingGuaranteedQuery
//         ->offset($offset)
//         ->limit($limit)
//         ->get();

//     /**
//      * 4. Group Loans I Am Guaranteed with Guarantors
//      */
//     $loansBeingGuaranteedGrouped = [];

//     foreach ($loansBeingGuaranteed as $loan) {
//         $loanId = $loan->loan_id_display;

//         if (!isset($loansBeingGuaranteedGrouped[$loanId])) {
//             $loansBeingGuaranteedGrouped[$loanId] = [
//                 'id' => $loan->loan_id,
//                 'loan_id' => $loanId,
//                 'type' => $loan->type,
//                 'amount' => $loan->amount,
//                 'totalRepayment' => $loan->totalRepayment,
//                 'duration' => $loan->duration,
//                 'memberId' => $loan->memberId,
//                 'member_name' => "{$loan->member_first_name} {$loan->member_last_name}",
//                 'member_image' => $loan->member_image,  // Member's profile image
//                 'approved' => $loan->approved,
//                 'approved_at' => $loan->approved_at,
//                 'created_at' => $loan->created_at,
//                 'updated_at' => $loan->updated_at,
//                 'guarantors' => [],
//             ];
//         }

//         $loansBeingGuaranteedGrouped[$loanId]['guarantors'][] = [
//             'guarantor_id' => $loan->guarantor_id,
//             'guarantor_name' => "{$loan->member_first_name} {$loan->member_last_name}",
//             'guarantor_image' => $loan->member_image,
//             'guaranteed_amount' => $loan->guaranteed_amount,
//             'accepted' => $loan->accepted,
//             'accepted_at' => $loan->accepted_at,
//             'created_at' => $loan->guarantor_created_at,
//             'updated_at' => $loan->guarantor_updated_at,
//         ];
//     }

//     // Convert the grouped loans into a list
//     $loansBeingGuaranteedList = array_values($loansBeingGuaranteedGrouped);

//     /**
//      * 5. Calculate Pagination Meta Information
//      */
//     $totalPagesGuaranteed = ceil($totalLoansGuaranteed / $limit);
//     $totalPagesBeingGuaranteed = ceil($totalLoansBeingGuaranteed / $limit);

//     /**
//      * 6. Return the Structured Data as JSON with Pagination Meta Information
//      */
//     return response()->json([
//         'loansGuaranteed' => $loansGuaranteedList,
//         'loansBeingGuaranteed' => $loansBeingGuaranteedList,
//         'pagination' => [
//             'current_page' => $page,
//             'per_page' => $limit,
//             'total_loans_guaranteed' => $totalLoansGuaranteed,
//             'total_pages_guaranteed' => $totalPagesGuaranteed,
//             'total_loans_being_guaranteed' => $totalLoansBeingGuaranteed,
//             'total_pages_being_guaranteed' => $totalPagesBeingGuaranteed,
//         ],
//     ]);
// }
function getGuaranteedLoans($userId, $perPage = 10)
{
    // Step 1: Get the loans guaranteed by the user with pagination
    $loans = DB::table('loan_guarantors')
        ->join('loans', 'loan_guarantors.loan_id', '=', 'loans.loanId')
        ->join('users as borrower', 'loans.memberId', '=', 'borrower.id')
        ->where('loan_guarantors.guarantor_id', $userId)
        ->select(
            'loans.loanId', 
            'loans.amount as borrowed_amount', 
            'loans.totalRepayment',
            'loans.duration',
           'loans.admin1_approved',
            'loans.admin2_approved',
            'loans.admin1_approved_at',
            'loans.admin2_approved_at',
            'loans.admin1_approved_by',
            'loans.admin2_approved_by',
            'loan_guarantors.created_at', 
            'loan_guarantors.updated_at', 
            'borrower.fname as borrower_fname', 
            'borrower.lname as borrower_lname', 
            'borrower.image as borrower_image'
        )
        ->distinct()
        ->paginate($perPage);  // Using paginate to limit the number of loans per page

    // Step 2: For each loan, get all guarantors with their details
    foreach ($loans as $loan) {
        $guarantors = DB::table('loan_guarantors')
            ->join('users', 'loan_guarantors.guarantor_id', '=', 'users.id')
            ->where('loan_guarantors.loan_id', $loan->loanId)
            ->select(
                'users.id as guarantor_id',
                'users.fname as guarantor_fname',
                'users.lname as guarantor_lname',
                'users.image as guarantor_image',
                'loan_guarantors.amount as guarantor_amount',
                'loan_guarantors.accepted as accepted',
                'loan_guarantors.accepted_at as accepted_at',
                'loan_guarantors.id as recid',
                'loan_guarantors.guarantor_id as guarantor_id'
                
            )
            ->get();

        // Attach the guarantors to the loan
        $loan->guarantors = $guarantors->map(function($guarantor) {
            return [
                'guarantor_fname' => $guarantor->guarantor_fname,
                'guarantor_lname' => $guarantor->guarantor_lname,
                'guarantor_image' => $guarantor->guarantor_image,
                'guarantor_amount' => $guarantor->guarantor_amount,
                'guarantor_accepted' => $guarantor->accepted,
                'guarantor_accepted_date' => $guarantor->accepted_at,
                'guarantor_record_id' => $guarantor->recid,
                'guarantor_id'=>$guarantor->guarantor_id
            ];
        });
    }

    return $loans;
}

public function acceptLoan($id)
{
    // Update the loan's accepted status and timestamp in the loan_guarantors table
    DB::table('loan_guarantors')
        ->where('id', $id)
        ->update([
            'accepted' => true,
            'accepted_at' => now(),
            'updated_at' => now(),
        ]);

    // Get the loan_guarantor details to access the guarantor_id and loan_id
    $loanGuarantor = DB::table('loan_guarantors')
        ->where('id', $id)
        ->first();

    // Retrieve the guarantor (user) using the guarantor_id from loan_guarantors
    $guarantor = DB::table('users')
        ->where('id', $loanGuarantor->guarantor_id)
        ->first();

    // Retrieve the loan details using the loan_id from loan_guarantors
    $loan = DB::table('loans')
        ->where('loanId', $loanGuarantor->loan_id)
        ->first();

    // Retrieve the loan applicant using the memberId from the loans table
    $applicant = DB::table('users')
        ->where('id', $loan->memberId)
        ->first();

    // Send notification to both the guarantor and the applicant
    //applicant
    $applicantMessage = new Request([
                'userID' =>  $applicant->id,
                'title' => 'Guarantor Acceptance',
                'message' => "Hi $applicant->fname,\n$guarantor->fname $guarantor->lname has accepted to guarantee your loan #$loan->loanId  for amount KES $loan->amount",
            ]);
            (new AlertsController())->sendAlert($applicantMessage);

    return response()->json(['message' => 'Loan accepted, and notifications sent to the applicant'],200);
}
    /**
     * Add a new payment record to the loanpayments table.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function addPayment(Request $request)
    {
        // Define validation rules
        $validator = Validator::make($request->all(), [
            'loanId' => 'required|exists:loans,loanId',
            'transactionCode' => 'required|string',
            'type' => 'required|string', // Adjust based on your types
            'amount' => 'required|numeric|min:0.01',
        
        ]);

        // Handle validation failures
        if ($validator->fails()) {
            return response()->json([
                'success' => false,
                'message' => 'Validation errors',
                'errors' => $validator->errors(),
            ], 422);
        }

        // Extract validated data
        $loanId = $request->input('loanId');
        $transactionCode = $request->input('transactionCode');
        $type = $request->input('type');
        $amount = $request->input('amount');
        $duration = $request->input('duration');

        // Insert the payment record
        try {
            DB::table('loanpayments')->insert([
                'loanId' => $loanId,
                'transactionCode' => $transactionCode,
                'type' => $type,
                'amount' => $amount,
                'approved' => false, // Set to true if approved by default
                'approved_at' => null,
                'created_at' => Carbon::now(),
                'updated_at' => Carbon::now(),
            ]);

            return response()->json([
                'success' => true,
                'message' => 'Payment record added successfully.',
            ], 200);
        } catch (\Exception $e) {
            // Log the error for debugging
            \Log::error('Error adding payment record: ' . $e->getMessage());

            return response()->json([
                'success' => false,
                'message' => 'Failed to add payment record.',
            ], 500);
        }
    }  
        //loan payments
       
    public function fetchLoanPayments(Request $request)
    {
        $page = $request->query('page', 1);
        $size = $request->query('size', 20);

        $loanPayments = DB::table('loanpayments')
            ->join('loans', 'loanpayments.loanId', '=', 'loans.loanId')
            ->join('users', 'loans.memberId', '=', 'users.id')
            ->select(
                'loanpayments.id',
                'loanpayments.loanId',
                'loanpayments.transactionCode',
                'loanpayments.type',
                'loanpayments.amount',
                'loanpayments.approved',
                'loanpayments.approved_at',
                'loanpayments.created_at',
                'loanpayments.updated_at',
                'users.id as userId',
                DB::raw("CONCAT(users.fname, ' ', users.lname) as userName"),
                'users.email as userEmail'
            )
            ->orderBy('loanpayments.created_at', 'desc')
            ->paginate($size, ['*'], 'page', $page);

        return response()->json([
            'success' => true,
            'data' => $loanPayments,
        ], 200);
    }

   public function updatePayment(Request $request, $id)
    {
        $loanPayment = DB::table('loanpayments')->where('id', $id)->first();

        if (!$loanPayment) {
            return response()->json([
                'success' => false,
                'message' => 'Loan payment not found.',
            ], 200);
        }

        $user = $request->user();
        $loan = DB::table('loans')->where('loanId', $loanPayment->loanId)->first();

        
        $validator = Validator::make($request->all(), [
            'amount' => 'required|numeric|min:0.01',
            'transactionCode' => 'required|string',
           'created_at' => 'required|date',
        ]);

        if ($validator->fails()) {
            return response()->json([
                'success'=> false,
                'message' => 'Validation errors',
                'errors' => $validator->errors(),
            ], 422);
        }

        $data = $validator->validated();

        DB::beginTransaction();

        try {
            $newAmount = $data['amount'];
            DB::table('loanpayments')
                ->where('id', $id)
                ->update([
                    'amount' => $newAmount,
                    'transactionCode' => $data['transactionCode'],
                    'created_at' => Carbon::parse($data['created_at']),
                    'updated_at' => Carbon::now(),
                ]);

            DB::commit();

            return response()->json([
                'success' => true,
                'message' => 'Loan payment updated successfully.',
            ], 200);

        } catch (\Exception $e) {
            DB::rollBack();
            \Log::error('Error updating loan payment: ' . $e->getMessage());

            return response()->json([
                'success' => false,
                'message' => 'Failed to update loan payment.',
            ], 500);
        }
    }

    public function deletePayment(Request $request, $id)
    {
        $loanPayment = DB::table('loanpayments')->where('id', $id)->first();

        if (!$loanPayment) {
            return response()->json([
                'success' => false,
                'message' => 'Loan payment not found.',
            ], 404);
        }

        DB::beginTransaction();

        try {
            DB::table('loanpayments')->where('id', $id)->delete();
            return response()->json([
                'success' => true,
                'message' => 'Loan payment deleted successfully.',
            ], 200);

        } catch (\Exception $e) {
            DB::rollBack();
            \Log::error('Error deleting loan payment: ' . $e->getMessage());

            return response()->json([
                'success' => false,
                'message' => 'Failed to delete loan payment.',
            ], 500);
        }
    }

// public function updatePaymentStatus(Request $request, $id)
// {
//     // Fetch the loan payment by its ID
//     $loanPayment = DB::table('loanpayments')->where('id', $id)->first();

//     // Check if the loan payment exists
//     if (!$loanPayment) {
//         return response()->json([
//             'success' => false,
//             'message' => 'Loan payment not found.',
//         ], 404);
//     }
//     $status = $request->input('status');

//     try {
//          // Update the loan payment status
//         DB::table('loanpayments')
//             ->where('id', $id)
//             ->update(['approved' => $status]);

//         return response()->json([
//             'success' => true,
//             'message' => 'Loan payment status updated successfully.',
//         ], 200);

//     } catch (\Exception $e) {
//       return response()->json([
//             'success' => false,
//             'message' => 'Failed to update loan payment status.',
//         ], 500);
//     }
// }
//
public function updatePaymentStatus(Request $request, $id)
{
     $status = $request->input('status');
    // Update the loan payment's approval status and timestamp in the loan_payments table
    DB::table('loanpayments')
        ->where('id', $id)
        ->update([
            'approved' => $status,
            'approved_at' => now(),
            'updated_at' => now(),
        ]);

    // Get the loan payment details to access the loan_id
    $loanPayment = DB::table('loanpayments')
        ->where('id', $id)
        ->first();

    // Check if loan payment exists
    if (!$loanPayment) {
        return response()->json(['message' => 'Loan payment not found'], 404);
    }

    // Retrieve the loan details using the loan_id from loan_payments
    $loan = DB::table('loans')
        ->where('loanId', $loanPayment->loanId)
        ->first();

    // Check if loan exists
    if (!$loan) {
        return response()->json(['message' => 'Loan not found'], 404);
    }

    // Retrieve the payer (user) using the memberId from the loans table
    $payer = DB::table('users')
        ->where('id', $loan->memberId)
        ->first();

    // Check if payer exists
    if (!$payer) {
        return response()->json(['message' => 'Payer not found'], 404);
    }

    // Send notification to the payer about the approved loan payment
    $payerMessage = new Request([
        'userID' => $payer->id,
        'title' => 'Loan Payment Approved',
        'message' => "Hi $payer->fname,\nYour loan payment of $loanPayment->amount  for loan #$loan->loanId has been approved.",
    ]);
    (new AlertsController())->sendAlert($payerMessage);

    return response()->json(['message' => 'Loan payment approved, and notification sent to the payer'], 200);
}


}

    





















