<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use App\Models\User;
use Illuminate\Support\Facades\DB;

class LoginController extends Controller
{
    public function login(Request $request)
    {
        // Validate input data with custom error messages
        $request->validate([
            'login' => 'required',    // Can be email or phone
            'password' => 'required'
        ], [
            'login.required' => 'The email or phone field is required.',
            'password.required' => 'The password field is required.',
        ]);

        // Determine if the user is trying to log in with email or phone number
        $loginType = filter_var($request->login, FILTER_VALIDATE_EMAIL) ? 'email' : 'phone';

        // Find the user by email or phone number
        $user = User::where($loginType, $request->login)->first();

        // Check if user exists and if the password matches
        if ($user && Hash::check($request->password, $user->password)) {
            // Create a Sanctum token for the user
            $token = $user->createToken('auth_token')->plainTextToken;

            // Return success response with token
            return response()->json([
                'status' => 'success',
                'message' => 'Login successful',
                'user' => $user,
                'token' => $token,  // Return token
                'token_type' => 'Bearer'
            ], 200);
        }

        // If login fails, return an error response with details
        return response()->json([
            'status' => 'error',
            'message' => 'Invalid email/phone or password. Please try again.'
        ], 401);
    }

public function listUsers($userId)
{
    // Query to select specific columns from users and calculate total contributions and shares from approved contributions
    $users = User::where('users.id', '!=', $userId)
                ->leftJoin('contributions', function($join) {
                    $join->on('users.id', '=', 'contributions.memberId')
                         ->where('contributions.approved', '=', true); // Only include approved contributions
                })
                ->select(
                    'users.id',                // User ID
                    'users.fname',             // First name
                    'users.lname',             // Last name
                    'users.memberNumber',      // Member number
                    'users.phone',             // Phone number
                    'users.email',             // Email
                    'users.IdNumber',          // ID Number
                    'users.role',              // User role
                    'users.active',            // Active status
                    'users.deviceid',          // Device ID
                    'users.created_at',        // Account creation date
                    'users.updated_at',        // Account last update date
                    'users.image',             // User profile image
                    DB::raw('SUM(contributions.amount) as total_contribution'), // Total contribution (approved only)
                    DB::raw('SUM(contributions.amount) as shares') // Total shares calculated from approved contributions
                )
                ->groupBy(
                    'users.id', 'users.fname', 'users.lname', 'users.memberNumber', 
                    'users.phone', 'users.email', 'users.IdNumber', 'users.role', 
                    'users.active', 'users.deviceid', 'users.created_at', 'users.updated_at',
                    'users.image' // Group by user fields
                )
                ->get();

    // Return the users along with total contributions and shares as a JSON response
    return response()->json([
        'status' => 'success',
        'users' => $users
    ], 200);
}



    public function getUserById($id)
{
    // Find the user by ID
    $user = User::find($id);

    // Check if the user exists
    if ($user) {
        // Return the user as a JSON response
        return response()->json([
            'status' => 'success',
            'user' => $user
        ], 200);
    } else {
        // Return an error response if the user is not found
        return response()->json([
            'status' => 'error',
            'message' => 'User not found'
        ], 404);
    }
}

}
